@@ -29,6 +29,7 @@
 
 #include "version.h"
 #include "nfsiostat.h"
+#include "count.h"
 #include "common.h"
 
 #ifdef USE_NLS
@@ -54,6 +55,7 @@ int flags = 0;		/* Flag for common options and system state */
 long interval = 0;
 char timestamp[64];
 
+struct sigaction alrm_act;
 
 /*
  ***************************************************************************
@@ -70,7 +72,7 @@ void usage(char *progname)
 
 #ifdef DEBUG
 	fprintf(stderr, _("Options are:\n"
-			  "[ --debuginfo ] [ -h ] [ -k | -m ] [ -t ] [ -V ]\n"));
+			  "[ -h ] [ -k | -m ] [ -t ] [ -V ] [ --debuginfo ]\n"));
 #else
 	fprintf(stderr, _("Options are:\n"
 			  "[ -h ] [ -k | -m ] [ -t ] [ -V ]\n"));
@@ -104,12 +106,11 @@ void set_output_unit(void)
  * SIGALRM signal handler.
  *
  * IN:
- * @sig	Signal number. Set to 0 for the first time, then to SIGALRM.
+ * @sig	Signal number.
  ***************************************************************************
  */
 void alarm_handler(int sig)
 {
-	signal(SIGALRM, alarm_handler);
 	alarm(interval);
 }
 
@@ -608,7 +609,7 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 		read_nfs_stat(curr);
 
 		/* Get time */
-		get_localtime(rectime);
+		get_localtime(rectime, 0);
 
 		/* Print results */
 		write_stats(curr, rectime);
@@ -729,7 +730,7 @@ int main(int argc, char **argv)
 	/* Init structures according to machine architecture */
 	io_sys_init();
 
-	get_localtime(&rectime);
+	get_localtime(&rectime, 0);
 
 	/* Get system name, release number and hostname */
 	uname(&header);
@@ -740,7 +741,10 @@ int main(int argc, char **argv)
 	printf("\n");
 
 	/* Set a handler for SIGALRM */
-	alarm_handler(0);
+	memset(&alrm_act, 0, sizeof(alrm_act));
+	alrm_act.sa_handler = (void *) alarm_handler;
+	sigaction(SIGALRM, &alrm_act, NULL);
+	alarm(interval);
 
 	/* Main loop */
 	rw_io_stat_loop(count, &rectime);
