@@ -1,6 +1,6 @@
 /*
  * rd_stats.c: Read system statistics
- * (C) 1999-2011 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2013 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -27,6 +27,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/vfs.h>
 #include <unistd.h>
 
 #include "common.h"
@@ -41,6 +42,42 @@
 #define _(string) (string)
 #endif
 
+/*
+ ***************************************************************************
+ * Replace octal codes in string with their corresponding characters.
+ *
+ * IN:
+ * @str		String to parse.
+ *
+ * OUT:
+ * @str		String with octal codes replaced with characters.
+ ***************************************************************************
+ */
+void oct2chr(char *str)
+{
+	int i = 0;
+	int j, len;
+	
+	len = strlen(str);
+	
+	while (i < len - 3) {
+		if ((str[i] == '\\') &&
+		    (str[i + 1] >= '0') && (str[i + 1] <= '3') &&
+		    (str[i + 2] >= '0') && (str[i + 2] <= '7') &&
+		    (str[i + 3] >= '0') && (str[i + 3] <= '7')) {
+			/* Octal code found */
+			str[i] = (str[i + 1] - 48) * 64 +
+			         (str[i + 2] - 48) * 8  +
+			         (str[i + 3] - 48);
+			for (j = i + 4; j <= len; j++) {
+				str[j - 3] = str[j];
+			}
+			len -= 3;
+		}
+		i++;
+	}
+}
+
 /*
  ***************************************************************************
  * Read CPU statistics and machine uptime.
@@ -84,7 +121,7 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 			 * (user, nice, etc.) among all proc. CPU usage is not reduced
 			 * to one processor to avoid rounding problems.
 			 */
-			sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu %llu",
+			sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu %llu %llu",
 			       &st_cpu->cpu_user,
 			       &st_cpu->cpu_nice,
 			       &st_cpu->cpu_sys,
@@ -93,14 +130,16 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 			       &st_cpu->cpu_hardirq,
 			       &st_cpu->cpu_softirq,
 			       &st_cpu->cpu_steal,
-			       &st_cpu->cpu_guest);
+			       &st_cpu->cpu_guest,
+			       &st_cpu->cpu_guest_nice);
 
 			/*
 			 * Compute the uptime of the system in jiffies (1/100ths of a second
 			 * if HZ=100).
 			 * Machine uptime is multiplied by the number of processors here.
 			 *
-			 * NB: Don't add cpu_guest because cpu_user already includes it.
+			 * NB: Don't add cpu_guest/cpu_guest_nice because cpu_user/cpu_nice
+			 * already include them.
 			 */
 			*uptime = st_cpu->cpu_user + st_cpu->cpu_nice    +
 				st_cpu->cpu_sys    + st_cpu->cpu_idle    +
@@ -117,7 +156,7 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 				 * (user, nice, etc) for current proc.
 				 * This is done only on SMP machines.
 				 */
-				sscanf(line + 3, "%d %llu %llu %llu %llu %llu %llu %llu %llu %llu",
+				sscanf(line + 3, "%d %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu",
 				       &proc_nb,
 				       &sc.cpu_user,
 				       &sc.cpu_nice,
@@ -127,7 +166,8 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 				       &sc.cpu_hardirq,
 				       &sc.cpu_softirq,
 				       &sc.cpu_steal,
-				       &sc.cpu_guest);
+				       &sc.cpu_guest,
+				       &sc.cpu_guest_nice);
 
 				if (proc_nb < (nbr - 1)) {
 					st_cpu_i = st_cpu + proc_nb + 1;
@@ -143,8 +183,8 @@ void read_stat_cpu(struct stats_cpu *st_cpu, int nbr,
 					 * Compute uptime reduced to one proc using proc#0.
 					 * Done if /proc/uptime was unavailable.
 					 *
-					 * NB: Don't add cpu_guest because cpu_user already
-					 * includes it.
+					 * NB: Don't add cpu_guest/cpu_guest_nice because cpu_user/cpu_nice
+					 * already include them.
 					 */
 					*uptime0 = sc.cpu_user + sc.cpu_nice  +
 						sc.cpu_sys     + sc.cpu_idle  +
@@ -347,6 +387,10 @@ void read_meminfo(struct stats_memory *st_memory)
 			/* Read the amount of free swap memory in kB */
 			sscanf(line + 9, "%lu", &st_memory->frskb);
 		}
+		else if (!strncmp(line, "Dirty:", 6)) {
+			/* Read the amount of dirty memory in kB */
+			sscanf(line + 6, "%lu", &st_memory->dirtykb);
+		}
 		else if (!strncmp(line, "Committed_AS:", 13)) {
 			/* Read the amount of commited memory in kB */
 			sscanf(line + 13, "%lu", &st_memory->comkb);
@@ -472,15 +516,14 @@ void read_diskstats_io(struct stats_io *st_io)
 	char line[256];
 	char dev_name[MAX_NAME_LEN];
 	unsigned int major, minor;
-	unsigned long rd_ios, wr_ios;
-	unsigned long long rd_sec, wr_sec;
+	unsigned long rd_ios, wr_ios, rd_sec, wr_sec;
 
 	if ((fp = fopen(DISKSTATS, "r")) == NULL)
 		return;
 
 	while (fgets(line, 256, fp) != NULL) {
 
-		if (sscanf(line, "%u %u %s %lu %*u %llu %*u %lu %*u %llu",
+		if (sscanf(line, "%u %u %s %lu %*u %lu %*u %lu %*u %lu",
 			   &major, &minor, dev_name,
 			   &rd_ios, &rd_sec, &wr_ios, &wr_sec) == 7) {
 			
@@ -491,9 +534,9 @@ void read_diskstats_io(struct stats_io *st_io)
 				 */
 				st_io->dk_drive      += rd_ios + wr_ios;
 				st_io->dk_drive_rio  += rd_ios;
-				st_io->dk_drive_rblk += (unsigned int) rd_sec;
+				st_io->dk_drive_rblk += rd_sec;
 				st_io->dk_drive_wio  += wr_ios;
-				st_io->dk_drive_wblk += (unsigned int) wr_sec;
+				st_io->dk_drive_wblk += wr_sec;
 			}
 		}
 	}
@@ -522,18 +565,16 @@ void read_diskstats_disk(struct stats_disk *st_disk, int nbr, int read_part)
 	char dev_name[MAX_NAME_LEN];
 	int dsk = 0;
 	struct stats_disk *st_disk_i;
-	unsigned int major, minor;
-	unsigned long rd_ios, wr_ios, rd_ticks, wr_ticks;
-	unsigned long tot_ticks, rq_ticks;
-	unsigned long long rd_sec, wr_sec;
+	unsigned int major, minor, rd_ticks, wr_ticks, tot_ticks, rq_ticks;
+	unsigned long rd_ios, wr_ios, rd_sec, wr_sec;
 
 	if ((fp = fopen(DISKSTATS, "r")) == NULL)
 		return;
 
 	while ((fgets(line, 256, fp) != NULL) && (dsk < nbr)) {
 
-		if (sscanf(line, "%u %u %s %lu %*u %llu %lu %lu %*u %llu"
-			   " %lu %*u %lu %lu",
+		if (sscanf(line, "%u %u %s %lu %*u %lu %u %lu %*u %lu"
+			   " %u %*u %u %u",
 			   &major, &minor, dev_name,
 			   &rd_ios, &rd_sec, &rd_ticks, &wr_ios, &wr_sec, &wr_ticks,
 			   &tot_ticks, &rq_ticks) == 11) {
@@ -707,8 +748,8 @@ void read_net_dev(struct stats_net_dev *st_net_dev, int nbr)
 			strncpy(iface, line, MINIMUM(pos, MAX_IFACE_LEN - 1));
 			iface[MINIMUM(pos, MAX_IFACE_LEN - 1)] = '\0';
 			sscanf(iface, "%s", st_net_dev_i->interface); /* Skip heading spaces */
-			sscanf(line + pos + 1, "%lu %lu %*u %*u %*u %*u %lu %lu %lu %lu "
-			       "%*u %*u %*u %*u %*u %lu",
+			sscanf(line + pos + 1, "%llu %llu %*u %*u %*u %*u %llu %llu %llu %llu "
+			       "%*u %*u %*u %*u %*u %llu",
 			       &st_net_dev_i->rx_bytes,
 			       &st_net_dev_i->rx_packets,
 			       &st_net_dev_i->rx_compressed,
@@ -755,8 +796,8 @@ void read_net_edev(struct stats_net_edev *st_net_edev, int nbr)
 			strncpy(iface, line, MINIMUM(pos, MAX_IFACE_LEN - 1));
 			iface[MINIMUM(pos, MAX_IFACE_LEN - 1)] = '\0';
 			sscanf(iface, "%s", st_net_edev_i->interface); /* Skip heading spaces */
-			sscanf(line + pos + 1, "%*u %*u %lu %lu %lu %lu %*u %*u %*u %*u "
-			       "%lu %lu %lu %lu %lu",
+			sscanf(line + pos + 1, "%*u %*u %llu %llu %llu %llu %*u %*u %*u %*u "
+			       "%llu %llu %llu %llu %llu",
 			       &st_net_edev_i->rx_errors,
 			       &st_net_edev_i->rx_dropped,
 			       &st_net_edev_i->rx_fifo_errors,
@@ -963,8 +1004,8 @@ void read_net_ip(struct stats_net_ip *st_net_ip)
 
 		if (!strncmp(line, "Ip:", 3)) {
 			if (sw) {
-				sscanf(line + 3, "%*u %*u %lu %*u %*u %lu %*u %*u "
-				       "%lu %lu %*u %*u %*u %lu %lu %*u %lu %*u %lu",
+				sscanf(line + 3, "%*u %*u %llu %*u %*u %llu %*u %*u "
+				       "%llu %llu %*u %*u %*u %llu %llu %*u %llu %*u %llu",
 				       &st_net_ip->InReceives,
 				       &st_net_ip->ForwDatagrams,
 				       &st_net_ip->InDelivers,
@@ -1009,8 +1050,8 @@ void read_net_eip(struct stats_net_eip *st_net_eip)
 
 		if (!strncmp(line, "Ip:", 3)) {
 			if (sw) {
-				sscanf(line + 3, "%*u %*u %*u %lu %lu %*u %lu %lu "
-				       "%*u %*u %lu %lu %*u %*u %*u %lu %*u %lu",
+				sscanf(line + 3, "%*u %*u %*u %llu %llu %*u %llu %llu "
+				       "%*u %*u %llu %llu %*u %*u %*u %llu %*u %llu",
 				       &st_net_eip->InHdrErrors,
 				       &st_net_eip->InAddrErrors,
 				       &st_net_eip->InUnknownProtos,
@@ -1324,34 +1365,34 @@ void read_net_ip6(struct stats_net_ip6 *st_net_ip6)
 	while (fgets(line, 128, fp) != NULL) {
 
 		if (!strncmp(line, "Ip6InReceives ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_ip6->InReceives6);
+			sscanf(line + 14, "%llu", &st_net_ip6->InReceives6);
 		}
 		else if (!strncmp(line, "Ip6OutForwDatagrams ", 20)) {
-			sscanf(line + 20, "%lu", &st_net_ip6->OutForwDatagrams6);
+			sscanf(line + 20, "%llu", &st_net_ip6->OutForwDatagrams6);
 		}
 		else if (!strncmp(line, "Ip6InDelivers ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_ip6->InDelivers6);
+			sscanf(line + 14, "%llu", &st_net_ip6->InDelivers6);
 		}
 		else if (!strncmp(line, "Ip6OutRequests ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_ip6->OutRequests6);
+			sscanf(line + 15, "%llu", &st_net_ip6->OutRequests6);
 		}
 		else if (!strncmp(line, "Ip6ReasmReqds ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_ip6->ReasmReqds6);
+			sscanf(line + 14, "%llu", &st_net_ip6->ReasmReqds6);
 		}
 		else if (!strncmp(line, "Ip6ReasmOKs ", 12)) {
-			sscanf(line + 12, "%lu", &st_net_ip6->ReasmOKs6);
+			sscanf(line + 12, "%llu", &st_net_ip6->ReasmOKs6);
 		}
 		else if (!strncmp(line, "Ip6InMcastPkts ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_ip6->InMcastPkts6);
+			sscanf(line + 15, "%llu", &st_net_ip6->InMcastPkts6);
 		}
 		else if (!strncmp(line, "Ip6OutMcastPkts ", 16)) {
-			sscanf(line + 16, "%lu", &st_net_ip6->OutMcastPkts6);
+			sscanf(line + 16, "%llu", &st_net_ip6->OutMcastPkts6);
 		}
 		else if (!strncmp(line, "Ip6FragOKs ", 11)) {
-			sscanf(line + 11, "%lu", &st_net_ip6->FragOKs6);
+			sscanf(line + 11, "%llu", &st_net_ip6->FragOKs6);
 		}
 		else if (!strncmp(line, "Ip6FragCreates ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_ip6->FragCreates6);
+			sscanf(line + 15, "%llu", &st_net_ip6->FragCreates6);
 		}
 	}
 	
@@ -1380,37 +1421,37 @@ void read_net_eip6(struct stats_net_eip6 *st_net_eip6)
 	while (fgets(line, 128, fp) != NULL) {
 
 		if (!strncmp(line, "Ip6InHdrErrors ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_eip6->InHdrErrors6);
+			sscanf(line + 15, "%llu", &st_net_eip6->InHdrErrors6);
 		}
 		else if (!strncmp(line, "Ip6InAddrErrors ", 16)) {
-			sscanf(line + 16, "%lu", &st_net_eip6->InAddrErrors6);
+			sscanf(line + 16, "%llu", &st_net_eip6->InAddrErrors6);
 		}
 		else if (!strncmp(line, "Ip6InUnknownProtos ", 19)) {
-			sscanf(line + 19, "%lu", &st_net_eip6->InUnknownProtos6);
+			sscanf(line + 19, "%llu", &st_net_eip6->InUnknownProtos6);
 		}
 		else if (!strncmp(line, "Ip6InTooBigErrors ", 18)) {
-			sscanf(line + 18, "%lu", &st_net_eip6->InTooBigErrors6);
+			sscanf(line + 18, "%llu", &st_net_eip6->InTooBigErrors6);
 		}
 		else if (!strncmp(line, "Ip6InDiscards ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_eip6->InDiscards6);
+			sscanf(line + 14, "%llu", &st_net_eip6->InDiscards6);
 		}
 		else if (!strncmp(line, "Ip6OutDiscards ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_eip6->OutDiscards6);
+			sscanf(line + 15, "%llu", &st_net_eip6->OutDiscards6);
 		}
 		else if (!strncmp(line, "Ip6InNoRoutes ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_eip6->InNoRoutes6);
+			sscanf(line + 14, "%llu", &st_net_eip6->InNoRoutes6);
 		}
 		else if (!strncmp(line, "Ip6OutNoRoutes ", 15)) {
-			sscanf(line + 15, "%lu", &st_net_eip6->OutNoRoutes6);
+			sscanf(line + 15, "%llu", &st_net_eip6->OutNoRoutes6);
 		}
 		else if (!strncmp(line, "Ip6ReasmFails ", 14)) {
-			sscanf(line + 14, "%lu", &st_net_eip6->ReasmFails6);
+			sscanf(line + 14, "%llu", &st_net_eip6->ReasmFails6);
 		}
 		else if (!strncmp(line, "Ip6FragFails ", 13)) {
-			sscanf(line + 13, "%lu", &st_net_eip6->FragFails6);
+			sscanf(line + 13, "%llu", &st_net_eip6->FragFails6);
 		}
 		else if (!strncmp(line, "Ip6InTruncatedPkts ", 19)) {
-			sscanf(line + 19, "%lu", &st_net_eip6->InTruncatedPkts6);
+			sscanf(line + 19, "%llu", &st_net_eip6->InTruncatedPkts6);
 		}
 	}
 	
@@ -1857,6 +1898,54 @@ void read_bus_usb_dev(struct stats_pwr_usb *st_pwr_usb, int nbr)
 	closedir(dir);
 }
 
+/*
+ ***************************************************************************
+ * Read filesystems statistics.
+ *
+ * IN:
+ * @st_filesystem	Structure where stats will be saved.
+ * @nbr			Total number of filesystems.
+ *
+ * OUT:
+ * @st_filesystem	Structure with statistics.
+ ***************************************************************************
+ */
+void read_filesystem(struct stats_filesystem *st_filesystem, int nbr)
+{
+	FILE *fp;
+	char line[256], fs_name[MAX_FS_LEN], mountp[128];
+	int fs = 0;
+	struct stats_filesystem *st_filesystem_i;
+	struct statfs buf;
+
+	if ((fp = fopen(MTAB, "r")) == NULL)
+		return;
+
+	while ((fgets(line, 256, fp) != NULL) && (fs < nbr)) {
+		if (line[0] == '/') {
+			
+			/* Read current filesystem name and mount point */
+			sscanf(line, "%71s %127s", fs_name, mountp);
+			
+			/* Replace octal codes */
+			oct2chr(mountp);
+			
+			if ((statfs(mountp, &buf) < 0) || (!buf.f_blocks))
+				continue;
+			
+			st_filesystem_i = st_filesystem + fs++;
+			st_filesystem_i->f_blocks = buf.f_blocks * buf.f_bsize;
+			st_filesystem_i->f_bfree  = buf.f_bfree * buf.f_bsize;
+			st_filesystem_i->f_bavail = buf.f_bavail * buf.f_bsize;
+			st_filesystem_i->f_files  = buf.f_files;
+			st_filesystem_i->f_ffree  = buf.f_ffree;
+			strcpy(st_filesystem_i->fs_name, fs_name);
+		}
+	}
+
+	fclose(fp);
+}
+
 /*
  ***************************************************************************
  * Read machine uptime, independently of the number of processors.
@@ -1886,393 +1975,3 @@ void read_uptime(unsigned long long *uptime)
 	fclose(fp);
 
 }
-
-/*
- ***************************************************************************
- * Count number of interrupts that are in /proc/stat file.
- *
- * RETURNS:
- * Number of interrupts, including total number of interrupts.
- ***************************************************************************
- */
-int get_irq_nr(void)
-{
-	FILE *fp;
-	char line[8192];
-	int in = 0;
-	int pos = 4;
-
-	if ((fp = fopen(STAT, "r")) == NULL)
-		return 0;
-
-	while (fgets(line, 8192, fp) != NULL) {
-
-		if (!strncmp(line, "intr ", 5)) {
-
-			while (pos < strlen(line)) {
-				in++;
-				pos += strcspn(line + pos + 1, " ") + 1;
-			}
-		}
-	}
-
-	fclose(fp);
-
-	return in;
-}
-
-/*
- ***************************************************************************
- * Find number of serial lines that support tx/rx accounting
- * in /proc/tty/driver/serial file.
- *
- * RETURNS:
- * Number of serial lines supporting tx/rx accouting.
- ***************************************************************************
- */
-int get_serial_nr(void)
-{
-	FILE *fp;
-	char line[256];
-	int sl = 0;
-
-	if ((fp = fopen(SERIAL, "r")) == NULL)
-		return 0;	/* No SERIAL file */
-
-	while (fgets(line, 256, fp) != NULL) {
-		/*
-		 * tx/rx statistics are always present,
-		 * except when serial line is unknown.
-		 */
-		if (strstr(line, "tx:") != NULL) {
-			sl++;
-		}
-	}
-
-	fclose(fp);
-
-	return sl;
-}
-
-/*
- ***************************************************************************
- * Find number of interfaces (network devices) that are in /proc/net/dev
- * file.
- *
- * RETURNS:
- * Number of network interfaces.
- ***************************************************************************
- */
-int get_iface_nr(void)
-{
-	FILE *fp;
-	char line[128];
-	int iface = 0;
-
-	if ((fp = fopen(NET_DEV, "r")) == NULL)
-		return 0;	/* No network device file */
-
-	while (fgets(line, 128, fp) != NULL) {
-		if (strchr(line, ':')) {
-			iface++;
-		}
-	}
-
-	fclose(fp);
-
-	return iface;
-}
-
-/*
- ***************************************************************************
- * Find number of devices and partitions available in /proc/diskstats.
- *
- * IN:
- * @count_part		Set to TRUE if devices _and_ partitions are to be
- *			counted.
- * @only_used_dev	When counting devices, set to TRUE if only devices
- *			with non zero stats must be counted.
- *
- * RETURNS:
- * Number of devices (and partitions).
- ***************************************************************************
- */
-int get_diskstats_dev_nr(int count_part, int only_used_dev)
-{
-	FILE *fp;
-	char line[256];
-	char dev_name[MAX_NAME_LEN];
-	int dev = 0, i;
-	unsigned long rd_ios, wr_ios;
-
-	if ((fp = fopen(DISKSTATS, "r")) == NULL)
-		/* File non-existent */
-		return 0;
-
-	/*
-	 * Counting devices and partitions is simply a matter of counting
-	 * the number of lines...
-	 */
-	while (fgets(line, 256, fp) != NULL) {
-		if (!count_part) {
-			i = sscanf(line, "%*d %*d %s %lu %*u %*u %*u %lu",
-				   dev_name, &rd_ios, &wr_ios);
-			if ((i == 2) || !is_device(dev_name, ACCEPT_VIRTUAL_DEVICES))
-				/* It was a partition and not a device */
-				continue;
-			if (only_used_dev && !rd_ios && !wr_ios)
-				/* Unused device */
-				continue;
-		}
-		dev++;
-	}
-
-	fclose(fp);
-
-	return dev;
-}
-
-/*
- ***************************************************************************
- * Get number of devices in /proc/diskstats.
- *
- * IN:
- * @f	Non zero (true) if disks *and* partitions should be counted, and
- *	zero (false) if only disks must be counted.
- *
- * RETURNS:
- * Number of devices.
- ***************************************************************************
- */
-int get_disk_nr(unsigned int f)
-{
-	int disk_nr;
-
-	/*
-	 * Partitions are taken into account by sar -d only with
-	 * kernels 2.6.25 and later.
-	 */
-	disk_nr = get_diskstats_dev_nr(f, CNT_USED_DEV);
-
-	return disk_nr;
-}
-
-/*
- ***************************************************************************
- * Count number of processors in /sys.
- *
- * RETURNS:
- * Number of processors (online and offline).
- * A value of 0 means that /sys was not mounted.
- * A value of N (!=0) means N processor(s) (cpu0 .. cpu(N-1)).
- ***************************************************************************
- */
-int get_sys_cpu_nr(void)
-{
-	DIR *dir;
-	struct dirent *drd;
-	struct stat buf;
-	char line[MAX_PF_NAME];
-	int proc_nr = 0;
-
-	/* Open relevant /sys directory */
-	if ((dir = opendir(SYSFS_DEVCPU)) == NULL)
-		return 0;
-
-	/* Get current file entry */
-	while ((drd = readdir(dir)) != NULL) {
-
-		if (!strncmp(drd->d_name, "cpu", 3) && isdigit(drd->d_name[3])) {
-			snprintf(line, MAX_PF_NAME, "%s/%s", SYSFS_DEVCPU, drd->d_name);
-			line[MAX_PF_NAME - 1] = '\0';
-			if (stat(line, &buf) < 0)
-				continue;
-			if (S_ISDIR(buf.st_mode)) {
-				proc_nr++;
-			}
-		}
-	}
-
-	/* Close directory */
-	closedir(dir);
-
-	return proc_nr;
-}
-
-/*
- ***************************************************************************
- * Count number of processors in /proc/stat.
- *
- * RETURNS:
- * Number of processors. The returned value is greater than or equal to the
- * number of online processors.
- * A value of 0 means one processor and non SMP kernel.
- * A value of N (!=0) means N processor(s) (0 .. N-1) with SMP kernel.
- ***************************************************************************
- */
-int get_proc_cpu_nr(void)
-{
-	FILE *fp;
-	char line[16];
-	int num_proc, proc_nr = -1;
-
-	if ((fp = fopen(STAT, "r")) == NULL) {
-		fprintf(stderr, _("Cannot open %s: %s\n"), STAT, strerror(errno));
-		exit(1);
-	}
-
-	while (fgets(line, 16, fp) != NULL) {
-
-		if (strncmp(line, "cpu ", 4) && !strncmp(line, "cpu", 3)) {
-			sscanf(line + 3, "%d", &num_proc);
-			if (num_proc > proc_nr) {
-				proc_nr = num_proc;
-			}
-		}
-	}
-
-	fclose(fp);
-
-	return (proc_nr + 1);
-}
-
-/*
- ***************************************************************************
- * Count the number of processors on the machine.
- * Try to use /sys for that, or /proc/stat if /sys doesn't exist.
- *
- * IN:
- * @max_nr_cpus	Maximum number of proc that sysstat can handle.
- *
- * RETURNS:
- * Number of processors.
- * 0: one proc and non SMP kernel.
- * 1: one proc and SMP kernel (NB: On SMP machines where all the CPUs but
- *    one have been disabled, we get the total number of proc since we use
- *    /sys to count them).
- * 2: two proc...
- ***************************************************************************
- */
-int get_cpu_nr(unsigned int max_nr_cpus)
-{
-	int cpu_nr;
-
-	if ((cpu_nr = get_sys_cpu_nr()) == 0) {
-		/* /sys may be not mounted. Use /proc/stat instead */
-		cpu_nr = get_proc_cpu_nr();
-	}
-
-	if (cpu_nr > max_nr_cpus) {
-		fprintf(stderr, _("Cannot handle so many processors!\n"));
-		exit(1);
-	}
-
-	return cpu_nr;
-}
-
-/*
- ***************************************************************************
- * Find number of interrupts available per processor (use
- * /proc/interrupts file or /proc/softirqs).
- *
- * IN:
- * @file		/proc file to read (interrupts or softirqs).
- * @max_nr_irqcpu       Maximum number of interrupts per processor that
- *                      sadc can handle.
- * @cpu_nr		Number of processors.
- *
- * RETURNS:
- * Number of interrupts per processor + a pre-allocation constant.
- ***************************************************************************
- */
-int get_irqcpu_nr(char *file, int max_nr_irqcpu, int cpu_nr)
-{
-	FILE *fp;
-	char *line = NULL;
-	unsigned int irq = 0;
-	int p;
-
-	if ((fp = fopen(file, "r")) == NULL)
-		return 0;       /* No interrupts file */
-
-	SREALLOC(line, char, INTERRUPTS_LINE + 11 * cpu_nr);
-
-	while ((fgets(line, INTERRUPTS_LINE + 11 * cpu_nr , fp) != NULL) &&
-	       (irq < max_nr_irqcpu)) {
-		p = strcspn(line, ":");
-		if ((p > 0) && (p < 16)) {
-			irq++;
-		}
-	}
-
-	fclose(fp);
-
-	if (line) {
-		free(line);
-	}
-
-	return irq;
-}
-
-/*
- ***************************************************************************
- * Count number of possible frequencies for CPU#0.
- *
- * RETURNS:
- * Number of frequencies.
- ***************************************************************************
- */
-int get_freq_nr(void)
-{
-	FILE *fp;
-	char filename[MAX_PF_NAME];
-	char line[128];
-	int freq = 0;
-
-	snprintf(filename, MAX_PF_NAME, "%s/cpu0/%s",
-		 SYSFS_DEVCPU, SYSFS_TIME_IN_STATE);
-	if ((fp = fopen(filename, "r")) == NULL)
-		return 0;	/* No time_in_state file for CPU#0 */
-
-	while (fgets(line, 128, fp) != NULL) {
-		freq++;
-	}
-
-	fclose(fp);
-
-	return freq;
-}
-
-/*
- ***************************************************************************
- * Count number of USB devices in /sys/bus/usb/devices.
- *
- * RETURNS:
- * Number of USB devices plugged into the system.
- * Don't count USB root hubs.
- * Return -1 if directory doesn't exist in sysfs.
- ***************************************************************************
- */
-int get_usb_nr(void)
-{
-	DIR *dir;
-	struct dirent *drd;
-	int usb = 0;
-
-	/* Open relevant /sys directory */
-	if ((dir = opendir(SYSFS_USBDEV)) == NULL)
-		return -1;
-
-	/* Get current file entry */
-	while ((drd = readdir(dir)) != NULL) {
-
-		if (isdigit(drd->d_name[0]) && !strchr(drd->d_name, ':')) {
-			usb++;
-		}
-	}
-
-	/* Close directory */
-	closedir(dir);
-
-	return usb;
-}
