@@ -1,6 +1,6 @@
 /*
  * rndr_stats.c: Funtions used by sadf to display statistics in selected format.
- * (C) 1999-2012 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2013 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -133,7 +133,11 @@ static void render(int isdb, char *pre, int rflags, const char *pptxt,
 			}
 		}
 		else {
-			printf(txt[isdb]);	/* No args */
+			/*
+			 * Additional NULL parameter below works around
+			 * fatal error when compiled with -Werror=format-security.
+			 */
+			printf(txt[isdb], NULL);	/* No args */
 		}
 	}
 
@@ -143,6 +147,9 @@ static void render(int isdb, char *pre, int rflags, const char *pptxt,
 	else if (rflags & PT_USESTR) {
 		printf("%s%s", seps[isdb], sval);
 	}
+	else if (rflags & PT_USERND) {
+		printf("%s%.0f", seps[isdb], dval);
+	}
 	else {
 		printf("%s%.2f", seps[isdb], dval);
 	}
@@ -204,11 +211,24 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					       NULL);
 				}
 				
-				render(isdb, pre, PT_NOFLAG,
-				       "all\t%%nice", NULL, NULL,
-				       NOVAL,
-				       ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
-				       NULL);
+				if (DISPLAY_CPU_DEF(a->opt_flags)) {
+					render(isdb, pre, PT_NOFLAG,
+					       "all\t%%nice", NULL, NULL,
+					       NOVAL,
+					       ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
+					       NULL);
+				}
+				else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+					render(isdb, pre, PT_NOFLAG,
+					       "all\t%%nice", NULL, NULL,
+					       NOVAL,
+					       (scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice) ?
+					       0.0 :
+					       ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
+							   scc->cpu_nice - scc->cpu_guest_nice,
+							   g_itv),
+					       NULL);
+				}				
 
 				if (DISPLAY_CPU_DEF(a->opt_flags)) {
 					render(isdb, pre, PT_NOFLAG,
@@ -257,6 +277,12 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					       NOVAL,
 					       ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
 					       NULL);
+
+					render(isdb, pre, PT_NOFLAG,
+					       "all\t%%gnice", NULL, NULL,
+					       NOVAL,
+					       ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
+					       NULL);
 				}
 
 				render(isdb, pre, pt_newlin,
@@ -271,7 +297,8 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 				/*
 				 * If the CPU is offline then it is omited from /proc/stat:
 				 * All the fields couldn't have been read and the sum of them is zero.
-				 * (Remember that guest time is already included in user mode.)
+				 * (Remember that guest/guest_nice times are already included in
+				 * user/nice modes.)
 				 */
 				if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
 				     scc->cpu_iowait  + scc->cpu_idle + scc->cpu_steal +
@@ -318,13 +345,26 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					       NULL);
 				}
 				
-				render(isdb, pre, PT_NOFLAG,
-				       "cpu%d\t%%nice", NULL, cons(iv, i - 1, NOVAL),
-				       NOVAL,
-				       !g_itv ?
-				       0.0 :
-				       ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
-				       NULL);
+				if (DISPLAY_CPU_DEF(a->opt_flags)) {
+					render(isdb, pre, PT_NOFLAG,
+					       "cpu%d\t%%nice", NULL, cons(iv, i - 1, NOVAL),
+					       NOVAL,
+					       !g_itv ?
+					       0.0 :
+					       ll_sp_value(scp->cpu_nice, scc->cpu_nice, g_itv),
+					       NULL);
+				}
+				else if (DISPLAY_CPU_ALL(a->opt_flags)) {
+					render(isdb, pre, PT_NOFLAG,
+					       "cpu%d\t%%nice", NULL, cons(iv, i - 1, NOVAL),
+					       NOVAL,
+					       (!g_itv ||
+					       ((scc->cpu_nice - scc->cpu_guest_nice) < (scp->cpu_nice - scp->cpu_guest_nice))) ?
+					       0.0 :
+					       ll_sp_value(scp->cpu_nice - scp->cpu_guest_nice,
+							   scc->cpu_nice - scc->cpu_guest_nice, g_itv),
+					       NULL);
+				}
 
 				if (DISPLAY_CPU_DEF(a->opt_flags)) {
 					render(isdb, pre, PT_NOFLAG,
@@ -387,6 +427,14 @@ __print_funct_t render_cpu_stats(struct activity *a, int isdb, char *pre,
 					       0.0 :
 					       ll_sp_value(scp->cpu_guest, scc->cpu_guest, g_itv),
 					       NULL);
+
+					render(isdb, pre, PT_NOFLAG,
+					       "cpu%d\t%%gnice", NULL, cons(iv, i - 1, NOVAL),
+					       NOVAL,
+					       !g_itv ?
+					       0.0 :
+					       ll_sp_value(scp->cpu_guest_nice, scc->cpu_guest_nice, g_itv),
+					       NULL);
 				}
 				
 				if (!g_itv) {
@@ -748,9 +796,13 @@ __print_funct_t render_memory_stats(struct activity *a, int isdb, char *pre,
 		       "-\tkbactive", NULL, NULL,
 		       smc->activekb, DNOVAL, NULL);
 
-		render(isdb, pre, PT_USEINT | pt_newlin,
+		render(isdb, pre, PT_USEINT,
 		       "-\tkbinact", NULL, NULL,
 		       smc->inactkb, DNOVAL, NULL);
+
+		render(isdb, pre, PT_USEINT | pt_newlin,
+		       "-\tkbdirty", NULL, NULL,
+		       smc->dirtykb, DNOVAL, NULL);
 	}
 	
 	if (DISPLAY_SWAP(a->opt_flags)) {
@@ -961,7 +1013,7 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 	int i, j;
 	struct stats_disk *sdc,	*sdp;
 	struct ext_disk_stats xds;
-	char *dev_name;
+	char *dev_name, *persist_dev_name;
 	int pt_newlin
 		= (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN);
 
@@ -979,14 +1031,24 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 		compute_ext_disk_stats(sdc, sdp, itv, &xds);
 
 		dev_name = NULL;
+		persist_dev_name = NULL;
 
-		if ((USE_PRETTY_OPTION(flags)) && (sdc->major == dm_major)) {
-			dev_name = transform_devmapname(sdc->major, sdc->minor);
+		if (DISPLAY_PERSIST_NAME_S(flags)) {
+			persist_dev_name = get_persistent_name_from_pretty(get_devname(sdc->major, sdc->minor, TRUE));
 		}
+		
+		if (persist_dev_name) {
+			dev_name = persist_dev_name;
+		}
+		else {
+			if ((USE_PRETTY_OPTION(flags)) && (sdc->major == dm_major)) {
+				dev_name = transform_devmapname(sdc->major, sdc->minor);
+			}
 
-		if (!dev_name) {
-			dev_name = get_devname(sdc->major, sdc->minor,
-					       USE_PRETTY_OPTION(flags));
+			if (!dev_name) {
+				dev_name = get_devname(sdc->major, sdc->minor,
+						       USE_PRETTY_OPTION(flags));
+			}
 		}
 
 		render(isdb, pre, PT_NOFLAG,
@@ -2733,3 +2795,90 @@ __print_funct_t render_pwr_usb_stats(struct activity *a, int isdb, char *pre,
 		       suc->product);
 	}
 }
+
+/*
+ ***************************************************************************
+ * Display filesystems statistics in selected format.
+ *
+ * IN:
+ * @a		Activity structure with statistics.
+ * @isdb	Flag, true if db printing, false if ppc printing.
+ * @pre		Prefix string for output entries
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time in jiffies.
+ ***************************************************************************
+ */
+__print_funct_t render_filesystem_stats(struct activity *a, int isdb, char *pre,
+					int curr, unsigned long long itv)
+{
+	int i;
+	struct stats_filesystem *sfc;
+
+	for (i = 0; i < a->nr; i++) {
+		sfc = (struct stats_filesystem *) ((char *) a->buf[curr] + i * a->msize);
+
+		if (!sfc->f_blocks)
+			/* Size of filesystem is null: We are at the end of the list */
+			break;
+
+		render(isdb, pre, PT_USERND,
+		       "%s\tMBfsfree",
+		       "%s",
+		       cons(sv, sfc->fs_name, NOVAL),
+		       NOVAL,
+		       (double) sfc->f_bfree / 1024 / 1024,
+		       NULL);
+
+		render(isdb, pre, PT_USERND,
+		       "%s\tMBfsused",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       NOVAL,
+		       (double) (sfc->f_blocks - sfc->f_bfree) / 1024 / 1024,
+		       NULL);
+
+		render(isdb, pre, PT_NOFLAG,
+		       "%s\t%%fsused",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       NOVAL,
+		       sfc->f_blocks ? SP_VALUE(sfc->f_bfree, sfc->f_blocks, sfc->f_blocks)
+				     : 0.0,
+		       NULL);
+
+		render(isdb, pre, PT_NOFLAG,
+		       "%s\t%%ufsused",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       NOVAL,
+		       sfc->f_blocks ? SP_VALUE(sfc->f_bavail, sfc->f_blocks, sfc->f_blocks)
+				     : 0.0,
+		       NULL);
+
+		render(isdb, pre, PT_USEINT,
+		       "%s\tIfree",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       sfc->f_ffree,
+		       NOVAL,
+		       NULL);
+
+		render(isdb, pre, PT_USEINT,
+		       "%s\tIused",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       sfc->f_files - sfc->f_ffree,
+		       NOVAL,
+		       NULL);
+
+		render(isdb, pre,
+		       (DISPLAY_HORIZONTALLY(flags) ? PT_NOFLAG : PT_NEWLIN),
+		       "%s\t%%Iused",
+		       NULL,
+		       cons(sv, sfc->fs_name, NOVAL),
+		       NOVAL,
+		       sfc->f_files ? SP_VALUE(sfc->f_ffree, sfc->f_files, sfc->f_files)
+				    : 0.0,
+		       NULL);
+	}
+}
