@@ -1,6 +1,6 @@
 /*
  * sysstat - sa_wrap.c: Functions used in activity.c
- * (C) 1999-2011 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2013 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -21,6 +21,7 @@
 
 #include "sa.h"
 #include "rd_stats.h"
+#include "count.h"
 #include "rd_sensors.h"
 
 extern unsigned int flags;
@@ -847,6 +848,28 @@ __read_funct_t wrap_read_bus_usb_dev(struct activity *a)
 	return;
 }
 
+/*
+ ***************************************************************************
+ * Read filesystem statistics.
+ *
+ * IN:
+ * @a	Activity structure.
+ *
+ * OUT:
+ * @a	Activity structure with statistics.
+ ***************************************************************************
+ */
+__read_funct_t wrap_read_filesystem(struct activity *a)
+{
+	struct stats_filesystem *st_filesystem
+		= (struct stats_filesystem *) a->_buf0;
+
+	/* Read filesystems from /etc/mtab */
+	read_filesystem(st_filesystem, a->nr);
+
+	return;
+}
+
 /*
  ***************************************************************************
  * Count number of interrupts that are in /proc/stat file.
@@ -1045,7 +1068,29 @@ __nr_t wrap_get_usb_nr(struct activity *a)
 
 	if ((n = get_usb_nr()) >= 0)
 		/* Return a positive number even if no USB devices have been found */
-		return (n + NR_USB_PREALLOC);
+		return n + NR_USB_PREALLOC;
 	
 	return 0;
 }
+
+/*
+ ***************************************************************************
+ * Get number of mounted filesystems from /etc/mtab. Don't take into account
+ * pseudo-filesystems.
+ *
+ * IN:
+ * @a	Activity structure.
+ *
+ * RETURNS:
+ * Number of filesystems + a pre-allocation constant.
+ ***************************************************************************
+ */
+__nr_t wrap_get_filesystem_nr(struct activity *a)
+{
+	__nr_t n = 0;
+
+	if ((n = get_filesystem_nr()) > 0)
+		return n + NR_FILESYSTEM_PREALLOC;
+
+	return 0;
+}
