@@ -1,6 +1,6 @@
 /*
  * sar: report system activity
- * (C) 1999-2011 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2013 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -25,6 +25,7 @@
 #include <unistd.h>
 #include <time.h>
 #include <errno.h>
+#include <signal.h>
 
 #include "version.h"
 #include "sa.h"
@@ -77,6 +78,9 @@ char *args[MAX_ARGV_NR];
 
 extern struct activity *act[];
 
+struct sigaction int_act;
+int sigint_caught = 0;
+
 /*
  ***************************************************************************
  * Print usage title message.
@@ -103,11 +107,12 @@ void usage(char *progname)
 {
 	print_usage_title(stderr, progname);
 	fprintf(stderr, _("Options are:\n"
-			  "[ -A ] [ -b ] [ -B ] [ -C ] [ -d ] [ -h ] [ -H ] [ -p ] [ -q ] [ -r ]\n"
-			  "[ -R ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -v ] [ -V ] [ -w ] [ -W ] [ -y ]\n"
+			  "[ -A ] [ -B ] [ -b ] [ -C ] [ -d ] [ -F ] [ -H ] [ -h ] [ -p ] [ -q ] [ -R ]\n"
+			  "[ -r ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -V ] [ -v ] [ -W ] [ -w ] [ -y ]\n"
 			  "[ -I { <int> [,...] | SUM | ALL | XALL } ] [ -P { <cpu> [,...] | ALL } ]\n"
 			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
-			  "[ -o [ <filename> ] | -f [ <filename> ] ]\n"
+			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
+			  "[ -f [ <filename> ] | -o [ <filename> ] | -[0-9]+ ]\n"
 			  "[ -i <interval> ] [ -s [ <hh:mm:ss> ] ] [ -e [ <hh:mm:ss> ] ]\n"));
 	exit(1);
 }
@@ -126,7 +131,8 @@ void display_help(char *progname)
 	printf(_("Main options and reports:\n"));
 	printf(_("\t-b\tI/O and transfer rate statistics\n"));
 	printf(_("\t-B\tPaging statistics\n"));
-	printf(_("\t-d\tBlock device statistics\n"));
+	printf(_("\t-d\tBlock devices statistics\n"));
+	printf(_("\t-F\tFilesystems statistics\n"));
 	printf(_("\t-H\tHugepages utilization statistics\n"));
 	printf(_("\t-I { <int> | SUM | ALL | XALL }\n"
 		 "\t\tInterrupts statistics\n"));
@@ -166,13 +172,28 @@ void display_help(char *progname)
 	printf(_("\t-S\tSwap space utilization statistics\n"));
 	printf(_("\t-u [ ALL ]\n"
 		 "\t\tCPU utilization statistics\n"));
-	printf(_("\t-v\tKernel table statistics\n"));
+	printf(_("\t-v\tKernel tables statistics\n"));
 	printf(_("\t-w\tTask creation and system switching statistics\n"));
 	printf(_("\t-W\tSwapping statistics\n"));
-	printf(_("\t-y\tTTY device statistics\n"));
+	printf(_("\t-y\tTTY devices statistics\n"));
 	exit(0);
 }
 
+/*
+ ***************************************************************************
+ * SIGINT signal handler.
+ *
+ * IN:
+ * @sig	Signal number.
+ ***************************************************************************
+ */
+void int_handler(int sig)
+{
+	sigint_caught = 1;
+	printf("\n");	/* Skip "^C" displayed on screen */
+
+}
+
 /*
  ***************************************************************************
  * Init some structures.
@@ -1037,6 +1058,12 @@ void read_stats(void)
 	/* Save the first stats collected. Will be used to compute the average */
 	copy_structures(act, id_seq, record_hdr, 2, 0);
 
+	/* Set a handler for SIGINT */
+	memset(&int_act, 0, sizeof(int_act));
+	int_act.sa_handler = (void *) int_handler;
+	int_act.sa_flags = SA_RESTART;
+	sigaction(SIGINT, &int_act, NULL);
+
 	/* Main loop */
 	do {
 
@@ -1064,7 +1091,13 @@ void read_stats(void)
 			count--;
 		}
 		if (count) {
-			curr ^= 1;
+			if (sigint_caught) {
+				/* SIGINT signal caught => Display average stats */
+				count = 0;
+			}
+			else {
+				curr ^= 1;
+			}
 		}
 	}
 	while (count);
@@ -1081,8 +1114,9 @@ void read_stats(void)
  */
 int main(int argc, char **argv)
 {
-	int i, opt = 1, args_idx = 2;
+	int i, rc, opt = 1, args_idx = 2;
 	int fd[2];
+	int day_offset = 0;
 	char from_file[MAX_FILE_LEN], to_file[MAX_FILE_LEN];
 	char ltemp[20];
 
@@ -1148,7 +1182,7 @@ int main(int argc, char **argv)
 				from_file[MAX_FILE_LEN - 1] = '\0';
 			}
 			else {
-				set_default_file(&rectime, from_file);
+				set_default_file(&rectime, from_file, day_offset);
 			}
 		}
 
@@ -1205,11 +1239,21 @@ int main(int argc, char **argv)
 				usage(argv[0]);
 			}
 		}
+		
+		else if ((strlen(argv[opt]) > 1) &&
+			 (strlen(argv[opt]) < 4) &&
+			 !strncmp(argv[opt], "-", 1) &&
+			 (strspn(argv[opt] + 1, DIGITS) == (strlen(argv[opt]) - 1))) {
+			day_offset = atoi(argv[opt++] + 1);
+		}
 
 		else if (!strncmp(argv[opt], "-", 1)) {
 			/* Other options not previously tested */
-			if (parse_sar_opt(argv, &opt, act, &flags, C_SAR)) {
-				usage(argv[0]);
+			if ((rc = parse_sar_opt(argv, &opt, act, &flags, C_SAR)) != 0) {
+				if (rc == 1) {
+					usage(argv[0]);
+				}
+				exit(1);
 			}
 			opt++;
 		}
@@ -1245,7 +1289,7 @@ int main(int argc, char **argv)
 	/* 'sar' is equivalent to 'sar -f' */
 	if ((argc == 1) ||
 	    ((interval < 0) && !from_file[0] && !to_file[0])) {
-		set_default_file(&rectime, from_file);
+		set_default_file(&rectime, from_file, day_offset);
 	}
 
 	if (tm_start.use && tm_end.use && (tm_end.tm_hour < tm_start.tm_hour)) {
@@ -1270,6 +1314,11 @@ int main(int argc, char **argv)
 	if (!interval && (from_file[0] || to_file[0])) {
 		usage(argv[0]);
 	}
+	
+	/* Cannot enter a day shift with -o option */
+	if (to_file[0] && day_offset) {
+		usage(argv[0]);
+	}
 
 	if (USE_PRETTY_OPTION(flags)) {
 		dm_major = get_devmap_major();
